"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.identifyActiveOrgByStatus = exports.OrgListUtil = void 0;
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
const path_1 = require("path");
const core_1 = require("@salesforce/core");
const lib_1 = require("@salesforce/kit/lib");
const utils_1 = require("./utils");
class OrgListUtil {
    static async retrieveLogger() {
        if (!OrgListUtil.logger) {
            OrgListUtil.logger = await core_1.Logger.child('OrgListUtil');
        }
        return OrgListUtil.logger;
    }
    /**
     * This method takes all locally configured orgs and organizes them into the following buckets:
     * { nonScratchOrgs: [{}], scratchOrgs: [{}] }
     * the scratchOrgInfo query.
     *
     * @param {string[]|null} userFilenames- an array of strings that are validated against the server.
     * @param {object} flags - the result of this.flags on an sfdx command
     */
    static async readLocallyValidatedMetaConfigsGroupedByOrgType(userFilenames, flags) {
        const contents = await OrgListUtil.readAuthFiles(userFilenames);
        const orgs = await OrgListUtil.groupOrgs(contents);
        // parallelize two very independent operations
        const [nonScratchOrgs, scratchOrgs] = await Promise.all([
            Promise.all(orgs.nonScratchOrgs.map(async (fields) => {
                var _a;
                if (flags.skipconnectionstatus) {
                    fields.connectedStatus = (_a = fields.connectedStatus) !== null && _a !== void 0 ? _a : 'Unknown';
                }
                else {
                    fields.connectedStatus = await OrgListUtil.determineConnectedStatusForNonScratchOrg(fields.username);
                }
                return fields;
            })),
            OrgListUtil.processScratchOrgs(orgs.scratchOrgs),
        ]);
        return {
            nonScratchOrgs,
            scratchOrgs,
        };
    }
    /**
     * Organizes the scratchOrgs by DevHub to optimize calls to retrieveScratchOrgInfoFromDevHub(), then calls reduceScratchOrgInfo()
     *
     * @param {ExtendedAuthFields[]} scratchOrgs- an array of strings that are validated against the server.
     * @returns the same scratch org list, but with updated information from the server.
     */
    static async processScratchOrgs(scratchOrgs) {
        const orgIdsGroupedByDevHub = scratchOrgs
            .filter((fields) => fields.devHubUsername)
            .reduce((accum, fields) => {
            var _a;
            accum[fields.devHubUsername] = [...((_a = accum[fields.devHubUsername]) !== null && _a !== void 0 ? _a : []), core_1.sfdc.trimTo15(fields.orgId)];
            return accum;
        }, {});
        const updatedContents = (await Promise.all(Object.entries(orgIdsGroupedByDevHub).map(async ([devHubUsername, orgIds]) => OrgListUtil.retrieveScratchOrgInfoFromDevHub(devHubUsername, orgIds)))).reduce((accumulator, iterator) => [...accumulator, ...iterator], []);
        return OrgListUtil.reduceScratchOrgInfo(updatedContents, scratchOrgs);
    }
    /**
     * Used to retrieve authInfo of the auth files
     *
     * @param fileNames All the filenames in the global hidden folder
     */
    static async readAuthFiles(fileNames) {
        const allAuths = await Promise.all(fileNames.map(async (fileName) => {
            try {
                const orgUsername = path_1.basename(fileName, '.json');
                return core_1.AuthInfo.create({ username: orgUsername });
            }
            catch (error) {
                const err = error;
                const logger = await OrgListUtil.retrieveLogger();
                logger.warn(`Problem reading file: ${fileName} skipping`);
                logger.warn(err.message);
            }
        }));
        // AuthInfos that have a userId are from user create, not an "org-level" auth.  Omit them
        return allAuths.filter((authInfo) => !!authInfo && !authInfo.getFields().userId);
    }
    /**
     * Helper to group orgs by {scratchOrg, nonScratchOrgs}
     * Also identifies which are default orgs from config
     *
     * @param {object} contents -The authinfo retrieved from the auth files
     * @param {string[]} excludeProperties - properties to exclude from the grouped configs ex. ['refreshToken', 'clientSecret']
     * @private
     */
    static async groupOrgs(authInfos) {
        const output = {
            scratchOrgs: [],
            nonScratchOrgs: [],
        };
        const config = (await core_1.ConfigAggregator.create()).getConfig();
        for (const authInfo of authInfos) {
            let currentValue;
            try {
                currentValue = OrgListUtil.removeRestrictedInfoFromConfig(authInfo.getFields(true));
            }
            catch (error) {
                const logger = await OrgListUtil.retrieveLogger();
                logger.warn(`Error decrypting ${authInfo.getUsername()}`);
                currentValue = OrgListUtil.removeRestrictedInfoFromConfig(authInfo.getFields());
            }
            const [alias, lastUsed] = await Promise.all([
                utils_1.getAliasByUsername(currentValue.username),
                core_1.fs.stat(path_1.join(core_1.Global.DIR, `${currentValue.username}.json`)),
            ]);
            currentValue.alias = alias;
            currentValue.lastUsed = lastUsed.atime;
            OrgListUtil.identifyDefaultOrgs(currentValue, config);
            if (currentValue.devHubUsername) {
                output.scratchOrgs.push(currentValue);
            }
            else {
                output.nonScratchOrgs.push(currentValue);
            }
        }
        return output;
    }
    /**
     * Helper utility to remove sensitive information from a scratch org auth config. By default refreshTokens and client secrets are removed.
     *
     * @param {*} config - scratch org auth object.
     * @param {string[]} properties - properties to exclude ex ['refreshToken', 'clientSecret']
     * @returns the config less the sensitive information.
     */
    static removeRestrictedInfoFromConfig(config, properties = ['refreshToken', 'clientSecret']) {
        return lib_1.omit(config, properties);
    }
    /** Identify the default orgs */
    static identifyDefaultOrgs(orgInfo, config) {
        if (config.defaultusername &&
            (orgInfo.username === config.defaultusername || orgInfo.alias === config.defaultusername)) {
            orgInfo.isDefaultUsername = true;
        }
        else if (config.defaultdevhubusername &&
            (orgInfo.username === config.defaultdevhubusername || orgInfo.alias === config.defaultdevhubusername)) {
            orgInfo.isDefaultDevHubUsername = true;
        }
    }
    static async retrieveScratchOrgInfoFromDevHub(devHubUsername, orgIdsToQuery) {
        const fields = [
            'CreatedDate',
            'Edition',
            'Status',
            'ExpirationDate',
            'Namespace',
            'OrgName',
            'CreatedBy.Username',
            'SignupUsername',
        ];
        try {
            const devHubOrg = await core_1.Org.create({ aliasOrUsername: devHubUsername });
            const conn = devHubOrg.getConnection();
            const data = await conn
                .sobject('ScratchOrgInfo')
                .find({ ScratchOrg: { $in: orgIdsToQuery } }, fields);
            return data.map((org) => ({
                ...org,
                devHubOrgId: devHubOrg.getOrgId(),
            }));
        }
        catch (err) {
            const logger = await OrgListUtil.retrieveLogger();
            logger.warn(`Error querying ${devHubUsername} for ${orgIdsToQuery.length} orgIds`);
            return [];
        }
    }
    static async reduceScratchOrgInfo(updatedContents, orgs) {
        // Reduce the information to key value pairs with signupUsername as key
        const contentMap = updatedContents.reduce((map, scratchOrgInfo) => {
            if (scratchOrgInfo) {
                map[scratchOrgInfo.SignupUsername] = scratchOrgInfo;
            }
            return map;
        }, {});
        for (const scratchOrgInfo of orgs) {
            const updatedOrgInfo = contentMap[scratchOrgInfo.username];
            if (updatedOrgInfo) {
                scratchOrgInfo.signupUsername = updatedOrgInfo.SignupUsername;
                scratchOrgInfo.createdBy = updatedOrgInfo.CreatedBy.Username;
                scratchOrgInfo.createdDate = updatedOrgInfo.CreatedDate;
                scratchOrgInfo.devHubOrgId = updatedOrgInfo.devHubOrgId;
                scratchOrgInfo.attributes = updatedOrgInfo.attributes;
                scratchOrgInfo.orgName = updatedOrgInfo.OrgName;
                scratchOrgInfo.edition = updatedOrgInfo.Edition;
                scratchOrgInfo.status = updatedOrgInfo.Status;
                scratchOrgInfo.expirationDate = updatedOrgInfo.ExpirationDate;
                scratchOrgInfo.isExpired = updatedOrgInfo.Status === 'Deleted';
                // the old toolbelt code always said Unknown.  I'd love to get rid of it.
                scratchOrgInfo.connectedStatus = 'Unknown';
            }
            else {
                const logger = await OrgListUtil.retrieveLogger();
                logger.warn(`Can't find ${scratchOrgInfo.username} in the updated contents`);
            }
        }
        return orgs;
    }
    /**
     * retrieves the connection info of an nonscratch org
     *
     * @param username The username used when the org was authenticated
     * @returns {Promise.<string>}
     */
    static async determineConnectedStatusForNonScratchOrg(username) {
        var _a, _b, _c;
        try {
            const org = await core_1.Org.create({ aliasOrUsername: username });
            if (org.getField(core_1.Org.Fields.DEV_HUB_USERNAME)) {
                return;
            }
            try {
                await org.refreshAuth();
                return 'Connected';
            }
            catch (err) {
                const error = err;
                const logger = await OrgListUtil.retrieveLogger();
                logger.trace(`error refreshing auth for org: ${org.getUsername()}`);
                logger.trace(error);
                return (_a = error.code) !== null && _a !== void 0 ? _a : error.message;
            }
        }
        catch (err) {
            const error = err;
            const logger = await OrgListUtil.retrieveLogger();
            logger.trace(`error refreshing auth for org: ${username}`);
            logger.trace(error);
            return (_c = (_b = error.code) !== null && _b !== void 0 ? _b : error.message) !== null && _c !== void 0 ? _c : 'Unknown';
        }
    }
}
exports.OrgListUtil = OrgListUtil;
const identifyActiveOrgByStatus = (org) => {
    return org.status === 'Active';
};
exports.identifyActiveOrgByStatus = identifyActiveOrgByStatus;
//# sourceMappingURL=orgListUtil.js.map